/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef NPP_DPPCOINCIDENCE_H
#define NPP_DPPCOINCIDENCE_H

#include "DPPLiveProcess.h"
#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/thread.hpp>
#include <stdint.h>

namespace npp {

class DPPCoincidence : public DPPLiveProcess {

public:
	//! Type of controller
	static const std::string TYPE;

	DPPCoincidence(const std::string& name);
	DPPCoincidence(const DPPCoincidence& controller);
	virtual ~DPPCoincidence();

	virtual void postConfiguration();

	virtual void start();
	virtual void stop();
	virtual void raz();

private:
	static const std::string NOMADCOINCIDENCE;
	static const char SHARED_MEMORY_NAME[];
	static const char SHARED_MEMORY_DATA_NAME[];
	static const char SHARED_MEMORY_MATRIX_NAME[];
	static const char SHARED_MEMORY_ADCRATES_NAME[];

	void resetData();
	void startLiveProcess();
	void stopLiveProcess();

	void executeLiveProcess();
	void executeRefreshProcess();

	Property<int32> dT;
	Property<int32> cleanDT;
	Property<int32> processBlockSize;
	Property<bool> test;
	Property<std::string> testFileName;
	ArrayProperty<int32> xData;
	Property<int32> coincidenceResolution;
	ArrayProperty<int32> coincidenceData;
	Property<int32> nOrder;
	Property<int32> refreshTimeS;
	Property<int32> one;

	DynamicProperty<int32> order;
	DynamicProperty<float64> globalRate;
	DynamicProperty<float64> globalCleanRate;
	DynamicProperty<float64> detectorRate;
	DynamicProperty<float64> detectorCleanRate;

	DynamicProperty<int32> ADCId;
	DynamicProperty<float64> ADCRate;

	float64 * _data;
	int32 * _xData;
	float64 * _ADCRates;
	boost::interprocess::managed_shared_memory * _segment;

	boost::mutex _mutex;
	boost::thread * _liveCoincidenceThread;

	bool _liveRunning;
	boost::thread * _liveRefreshThread;

	static const int32 MAX_ADC = 128;
};

}

#endif
