/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RemoteDPPContext.h"

namespace npp {

using namespace std;
using namespace lstdpp128;

const std::string RemoteDPPContext::TYPE = "remote_dpp_context";

RemoteDPPContext::RemoteDPPContext(const std::string& name) :
	ExperimentController(name) {

	acquisitionController.init(this, "acquisition_controller");
}

RemoteDPPContext::~RemoteDPPContext() {
}

void RemoteDPPContext::postConfiguration() {

	// Warning: we suppose that the list mode block is ordered like the ADC controllers
	releaseListModeContext(listModeContext);

	int32 nbCrates = 0;

	for (int32 adc = 0; adc < acquisitionController->nbAdcControllers(); ++adc) {
		int32 crate = acquisitionController->adcController[adc]->crateNumber();
		if (crate >= nbCrates) {
			nbCrates = crate + 1;
		}
	}

	listModeContext.crateBoard.nbCrates = nbCrates;
	listModeContext.crateBoard.crates = new Crate[nbCrates];

	for (int32 crate = 0; crate < nbCrates; ++crate) {
		int32 nbBoards = 0;

		for (int32 board = 0; board < acquisitionController->nbAdcControllers(); ++board) {
			if (board >= nbBoards) {
				nbBoards = board + 1;
			}
		}

		listModeContext.crateBoard.crates[crate].nbBoards = nbBoards;
		listModeContext.crateBoard.crates[crate].boards = new Board[nbBoards];

		for (int32 board = 0; board < acquisitionController->nbAdcControllers(); ++board) {
			listModeContext.crateBoard.crates[0].boards[board].boardType = (BoardType)acquisitionController->adcController[board]->cardType();
			listModeContext.crateBoard.crates[0].boards[board].crate = crate;
			listModeContext.crateBoard.crates[0].boards[board].eventType = 0;
			listModeContext.crateBoard.crates[0].boards[board].nbChannels = acquisitionController->adcController[board]->nbChannels();
		}
	}

	cout << "DPP context" << endl;
	cout << listModeContext << endl;
}

}

