/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef QE65000CONTROLLER_H_
#define QE65000CONTROLLER_H_

#include <Controller.h>
#include "drivers/oceanoptics/QE65000Driver.h"

using namespace oceanoptics;

namespace spectrometer {

class QE65000Controller : public ExperimentController , public controller::Stoppable {

public:

	static const std::string TYPE;

	static const std::string IDLE;
	static const std::string SEQUENCE;
	static const std::string COUNT_SINGLE;
	static const std::string COUNT_SWITCHER;

	Property<int32> integrationTime;
	Property<int32> spectroStatus;
	Property<std::string> spectroMode;
	Property<std::string> plotName;
	Property<int32> numor;
	Property<std::string> spectroStatusMessage;
	Property<std::string> spectroOpenErrorMessage;
	Property<int32> scansToAverage;
	Property<int32> size;
	ArrayProperty<float64> xData;
	ArrayProperty<float64> yData;

	Property<std::string> mode; //this property is used for nexus files to indicate how was used the spectro when saving the data (when being used with a count)
								// possible values are: SEQUENCE, COUNT_SINGLE, COUNT_SWITCHER and idle.

	QE65000Controller(const std::string& name);
	virtual ~QE65000Controller();

	virtual void postConfiguration();

	virtual void refreshIntegrationTime(int32 integrationTime);

	//Important, this allows the connexion with the driver!
	DriverPtr<QE65000Driver> spectroDriver;

	virtual void start();
	virtual void stop();

	bool isConnected();

	void updateProgression();
	void updateStatus();
	void updateSize();
	void updateOpenError();

	void setPlotName(int32 plotId);

private:
	bool m_StopActivated;

};



}


#endif
