/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DFK31AU03DRIVER_H
#define DFK31AU03DRIVER_H

#include <Driver.h>
#include "controllers/common/acquisition/gate/TimeGateCommon.h"
#include "controllers/common/acquisition/mode/NoTofAcquisitionCommon.h"
#include "controllers/common/acquisition/detector/DetectorCommon.h"

namespace dfk41au03 {

/**
 * Global Implementation of the vme card called Dfk31Au03 made in Ill. It needs a board called Men A201.
 */
class Dfk31Au03Driver: public acquisition::NoTofAcquisitionCommon, public acquisition::DetectorCommon {

	friend class RealDfk31Au03Driver;
	friend class PerfectDfk31Au03Driver;
	friend class SimulatedDfk31Au03Driver;

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	Dfk31Au03Driver(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~Dfk31Au03Driver();

	/*!
	 * \brief Method called for executing a command
	 *
	 * \param[in] command the command to apply on the controller
	 */
	virtual void execute(const std::string& aCommand);

	/*!
	 * \brief Calculate the progression state.
	 */
	Property<string> RGBData;
	Property<int32> height;
	Property<int32> width;
	Property<int32> bitPerPixel;
	Property<string> pixelFormat;
	ArrayProperty<int32> PNGData;

private:

	/*!
	 * \brief Method called before changing the data size property value
	 * \param[in] value the property value
	 * \throws CannotSetValue the value isn't corrected, or property couldn't be changed
	 */
	virtual void refreshModeProperty(bool value) throw (CannotSetValue);

	/*!
	 * \brief compute global acquisition status
	 * \param[in] value the driver status
	 */
	virtual void computeAcquisitionStatus(int32 value);

	/*!
	 * \brief compute global detector status
	 * \param[in] value the driver status
	 */
	virtual void computeDetectorStatus(int32 value);

	/**
	 * flag for synchronise read
	 */
	bool m_SynchroniseReadActivated;
	bool m_SynchroniseReadTerminated;

	int32* m_pngarray;

};

}
#endif //DFK31AU03DRIVER_H
