/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "QE65000Driver.h"
#include "RealQE65000Driver.h"
#include "SimulatedQE65000Driver.h"
#include "QE65000State.h"
#include "drivers/global/DriversCommands.h"

namespace oceanoptics {

const std::string QE65000Driver::TYPE = "QE65000";
const int32 QE65000Driver::PLUGGED = 1;
const int32 QE65000Driver::UNPLUGGED = 0;

QE65000Driver::QE65000Driver(const std::string& name) : DeviceDriver(name) {

	registerParentFunction(NONE_FUNCTION); //no parent function

	//Register states --> up to now only RealState
	registerStates(new RealQE65000Driver(this), new SimulatedQE65000Driver(this), new SimulatedQE65000Driver(this));

	//Init Properties
	integrationTime.init(this, SAVE, "integration_time");
	isActive.init(this, SAVE, "isActive");
	openError.init(this, NOSAVE, "open_error");
	xData.init(this, NOSAVE, "x_data");
	yData.init(this, NOSAVE, "y_data");
	size.init(this, SAVE, "size");

	//Init functions
	registerFunction(NONE_FUNCTION);

	initCommand(driver::INIT_COMMAND);

	//Root device -> it is directly plugged in port USB of the PC
	updateValue(DEVICE_TYPE_DEVICE_CONTAINER,std::string(ROOT_DEVICE_TYPE_DEVICE_CONTAINER));

	//Register the Spy and Observer commands necessary to do the updates.
	registerSpyCommand(driver::STATUS_COMMAND, 1);
	registerObserverCommand(driver::STATUS_COMMAND, 20);

}

QE65000Driver::~QE65000Driver() {
}

void QE65000Driver::execute(const std::string& aCommand) {
	//Get the state where to execute the accion
	QE65000State* currentState = dynamic_cast<QE65000State*> (getCurrentState());

	if (aCommand == driver::START_COMMAND) {
		//begin reading
		currentState->start();

	} else if (aCommand == driver::STOP_COMMAND) {
		currentState->stop();

	} else if (aCommand == driver::STATUS_COMMAND) {
		currentState->readStatus();

	} else if (aCommand == driver::INIT_COMMAND) {
		// init command
		currentState->init();
	}
}

}
